@@ -1,36 +1,38 @@
 import json
 import os
 import shutil
-import sys
 from pathlib import Path
 from typing import Dict, Iterable, List, Optional, Sequence
 
 import click
+import tomlkit
 from pip._vendor.pkg_resources import safe_name
+from resolvelib.resolvers import ResolutionImpossible, ResolutionTooDeep
 
 from pdm.cli.utils import (
     check_project_file,
     find_importable_files,
+    find_python_in_path,
     format_lockfile,
-    format_toml,
+    format_resolution_impossible,
     save_version_specifiers,
     set_env_in_reg,
 )
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
+from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
 from pdm.installers.installers import format_dist
 from pdm.iostream import LOCK, stream
 from pdm.models.builders import EnvBuilder
 from pdm.models.candidates import Candidate
+from pdm.models.in_process import get_python_version
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import get_specifier
 from pdm.project import Project
 from pdm.resolver import resolve
-from pdm.utils import get_python_version
+from pdm.utils import get_python_version_string, setdefault
 
-PEP582_PATH = os.path.join(
-    os.path.dirname(sys.modules[__name__.split(".")[0]].__file__), "pep582"
-)
+PEP582_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pep582")
 
 
 def do_lock(
@@ -62,11 +64,25 @@ def do_lock(
         ) as spin:
             reporter = project.get_reporter(requirements, tracked_names, spin)
             resolver = project.core.resolver_class(provider, reporter)
-            mapping, dependencies, summaries = resolve(
-                resolver, requirements, project.environment.python_requires
-            )
-            data = format_lockfile(mapping, dependencies, summaries)
-            spin.succeed(f"{LOCK} Lock successful")
+            try:
+                mapping, dependencies, summaries = resolve(
+                    resolver, requirements, project.environment.python_requires
+                )
+            except ResolutionTooDeep:
+                spin.fail(f"{LOCK} Lock failed")
+                stream.echo(
+                    "The dependency resolution exceeds the maximum loop depth of 100, "
+                    "there may be some circular dependencies in your project.",
+                    err=True,
+                )
+                raise
+            except ResolutionImpossible as err:
+                spin.fail(f"{LOCK} Lock failed")
+                stream.echo(format_resolution_impossible(err), err=True)
+                raise
+            else:
+                data = format_lockfile(mapping, dependencies, summaries)
+                spin.succeed(f"{LOCK} Lock successful")
     project.write_lockfile(data)
 
     return mapping
@@ -259,31 +275,27 @@ def do_remove(
     if not packages:
         raise PdmUsageError("Must specify at least one package to remove.")
     section = "dev" if dev else section or "default"
-    toml_section = f"{section}-dependencies" if section != "default" else "dependencies"
-    if toml_section not in project.tool_settings:
-        raise ProjectError(
-            f"No such section {stream.yellow(toml_section)} in pyproject.toml."
-        )
-    deps = project.tool_settings[toml_section]
+    if section not in list(project.iter_sections()):
+        raise ProjectError(f"No {section} dependencies given in pyproject.toml.")
+
+    deps = project.get_pyproject_dependencies(section)
     stream.echo(
         f"Removing packages from {section} dependencies: "
         + ", ".join(str(stream.green(name, bold=True)) for name in packages)
     )
     for name in packages:
-        matched_name = next(
-            filter(
-                lambda k: safe_name(k).lower() == safe_name(name).lower(),
-                deps.keys(),
-            ),
-            None,
+        req = parse_requirement(name)
+        matched_indexes = sorted(
+            (i for i, r in enumerate(deps) if req.matches(r, False)), reverse=True
         )
-        if not matched_name:
+        if not matched_indexes:
             raise ProjectError(
                 "{} does not exist in {} dependencies.".format(
                     stream.green(name, bold=True), section
                 )
             )
-        del deps[matched_name]
+        for i in matched_indexes:
+            del deps[i]
 
     project.write_pyproject()
     do_lock(project, "reuse")
@@ -365,30 +377,27 @@ def do_init(
     python_requires: str = "",
 ) -> None:
     """Bootstrap the project and create a pyproject.toml"""
-    import tomlkit
-
     data = {
-        "tool": {
-            "pdm": {
-                "name": name,
-                "version": version,
-                "description": "",
-                "author": f"{author} <{email}>",
-                "license": license,
-                "homepage": "",
-                "dependencies": tomlkit.table(),
-                "dev-dependencies": tomlkit.table(),
-            }
+        "project": {
+            "name": name,
+            "version": version,
+            "description": "",
+            "authors": array_of_inline_tables([{"name": author, "email": email}]),
+            "license": make_inline_table({"text": license}),
+            "urls": {"homepage": ""},
+            "dependencies": make_array([], True),
+            "dev-dependencies": make_array([], True),
+            "requires-python": python_requires,
+            "dynamic": ["classifiers"],
         },
         "build-system": {"requires": ["pdm-pep517"], "build-backend": "pdm.pep517.api"},
     }
     if python_requires and python_requires != "*":
         get_specifier(python_requires)
-        data["tool"]["pdm"]["python_requires"] = python_requires
     if not project.pyproject:
         project._pyproject = data
     else:
-        project._pyproject.setdefault("tool", {})["pdm"] = data["tool"]["pdm"]
+        project._pyproject["project"] = data["project"]
         project._pyproject["build-system"] = data["build-system"]
     project.write_pyproject()
 
@@ -401,25 +410,28 @@ def do_use(project: Project, python: str, first: bool = False) -> None:
 
     if python and not all(c.isdigit() for c in python.split(".")):
         if Path(python).exists():
-            python_path = Path(python).absolute().as_posix()
+            python_path = find_python_in_path(python)
         else:
             python_path = shutil.which(python)
         if not python_path:
             raise NoPythonVersion(f"{python} is not a valid Python.")
-        python_version = get_python_version(python_path, True)
+        python_version, is_64bit = get_python_version(python_path, True)
     else:
         finder = pythonfinder.Finder()
         pythons = []
         args = [int(v) for v in python.split(".") if v != ""]
         for i, entry in enumerate(finder.find_all_python_versions(*args)):
-            python_version = get_python_version(entry.path.as_posix(), True)
-            pythons.append((entry.path.as_posix(), python_version))
+            python_version, is_64bit = get_python_version(entry.path.as_posix(), True)
+            pythons.append((entry.path.as_posix(), python_version, is_64bit))
         if not pythons:
             raise NoPythonVersion(f"Python {python} is not available on the system.")
 
         if not first and len(pythons) > 1:
-            for i, (path, python_version) in enumerate(pythons):
-                stream.echo(f"{i}. {stream.green(path)} ({python_version})")
+            for i, (path, python_version, is_64bit) in enumerate(pythons):
+                stream.echo(
+                    f"{i}. {stream.green(path)} "
+                    f"({get_python_version_string(python_version, is_64bit)})"
+                )
             selection = click.prompt(
                 "Please select:",
                 type=click.Choice([str(i) for i in range(len(pythons))]),
@@ -428,7 +440,7 @@ def do_use(project: Project, python: str, first: bool = False) -> None:
             )
         else:
             selection = 0
-        python_path, python_version = pythons[int(selection)]
+        python_path, python_version, is_64bit = pythons[int(selection)]
 
     if not project.python_requires.contains(python_version):
         raise NoPythonVersion(
@@ -437,7 +449,8 @@ def do_use(project: Project, python: str, first: bool = False) -> None:
         )
     stream.echo(
         "Using Python interpreter: {} ({})".format(
-            stream.green(python_path), python_version
+            stream.green(python_path),
+            get_python_version_string(python_version, is_64bit),
         )
     )
     old_path = project.config.get("python.path")
@@ -456,13 +469,14 @@ def do_info(
 ) -> None:
     """Show project information."""
     python_path = project.environment.python_executable
-    python_version = get_python_version(python_path, True)
+    python_version, is_64bit = get_python_version(python_path, True)
     if not python and not show_project and not env:
         rows = [
             (stream.cyan("PDM version:", bold=True), project.core.version),
             (
                 stream.cyan("Python Interpreter:", bold=True),
-                python_path + f" ({python_version})",
+                python_path
+                + f" ({get_python_version_string(python_version, is_64bit)})",
             ),
             (stream.cyan("Project Root:", bold=True), project.root.as_posix()),
         ]
@@ -495,16 +509,28 @@ def do_import(project: Project, filename: str, format: Optional[str] = None) ->
             )
     else:
         key = format
-    tool_settings = FORMATS[key].convert(project, filename)
-    format_toml(tool_settings)
+    project_data, settings = FORMATS[key].convert(project, filename)
+    pyproject = project.pyproject or tomlkit.document()
 
-    if not project.pyproject_file.exists():
-        project.pyproject = {"tool": {"pdm": {}}}
-    project.tool_settings.update(tool_settings)
-    project.pyproject["build-system"] = {
+    if "tool" not in pyproject or "pdm" not in pyproject["tool"]:
+        setdefault(pyproject, "tool", {})["pdm"] = tomlkit.table()
+
+    pyproject["tool"]["pdm"].update(settings)
+
+    if "project" not in pyproject:
+        pyproject.add("project", tomlkit.table())
+        pyproject["project"].add(tomlkit.comment("PEP 621 project metadata"))
+        pyproject["project"].add(
+            tomlkit.comment("See https://www.python.org/dev/peps/pep-0621/")
+        )
+
+    pyproject["project"].update(project_data)
+
+    pyproject["build-system"] = {
         "requires": ["pdm-pep517"],
-        "build-backend": ["pdm.pep517.api"],
+        "build-backend": "pdm.pep517.api",
     }
+    project.pyproject = pyproject
     project.write_pyproject()
 
 
@@ -558,3 +584,28 @@ def print_pep582_command(shell: str = "AUTO"):
             "via `--pep582 <SHELL>`"
         )
     stream.echo(result)
+
+
+def migrate_pyproject(project: Project):
+    """Migrate the legacy pyproject format to PEP 621"""
+
+    if (
+        not project.pyproject_file.exists()
+        or not FORMATS["legacy"].check_fingerprint(project, project.pyproject_file)
+        or "project" in project.pyproject
+    ):
+        return
+
+    stream.echo(
+        stream.yellow("Legacy [tool.pdm] metadata detected, migrating to PEP 621..."),
+        err=True,
+    )
+    do_import(project, project.pyproject_file, "legacy")
+    stream.echo(
+        stream.green("pyproject.toml")
+        + stream.yellow(
+            " has been migrated to PEP 621 successfully. "
+            "Now you can safely delete the legacy metadata under [tool.pdm] table."
+        ),
+        err=True,
+    )
