@@ -2,17 +2,20 @@ from __future__ import annotations
 
 import argparse
 import os
+import re
 import sys
 from collections import ChainMap
-from typing import TYPE_CHECKING, Optional
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 import cfonts
 import tomlkit
 from packaging.specifiers import SpecifierSet
 from resolvelib.structs import DirectedGraph
 
-from pdm.exceptions import ProjectError
+from pdm.exceptions import NoPythonVersion, ProjectError
 from pdm.formats import FORMATS
+from pdm.formats.base import make_inline_table
 from pdm.iostream import stream
 from pdm.models.environment import WorkingSet
 from pdm.models.requirements import Requirement, strip_extras
@@ -20,8 +23,9 @@ from pdm.models.specifiers import bump_version, get_specifier
 from pdm.project import Project
 
 if TYPE_CHECKING:
-    from pathlib import Path
-    from typing import Dict, Iterable, Tuple
+    from typing import Dict, Iterable, List, Optional, Tuple
+
+    from resolvelib.resolvers import RequirementInformation, ResolutionImpossible
 
     from pdm.models.candidates import Candidate
 
@@ -332,9 +336,7 @@ def format_lockfile(mapping, fetched_dependencies, summary_collection):
         for r in fetched_dependencies[k].values():
             name, req = r.as_req_dict()
             if getattr(req, "items", None) is not None:
-                inline = tomlkit.inline_table()
-                inline.update(req)
-                deps.add(name, inline)
+                deps.add(name, make_inline_table(req))
             else:
                 deps.add(name, req)
         if len(deps) > 0:
@@ -345,8 +347,7 @@ def format_lockfile(mapping, fetched_dependencies, summary_collection):
             array = tomlkit.array()
             array.multiline(True)
             for filename, hash_value in v.hashes.items():
-                inline = tomlkit.inline_table()
-                inline.update({"file": filename, "hash": hash_value})
+                inline = make_inline_table({"file": filename, "hash": hash_value})
                 array.append(inline)
             if array:
                 file_hashes.add(key, array)
@@ -383,25 +384,13 @@ def save_version_specifiers(
 
 def check_project_file(project: Project) -> None:
     """Check the existence of the project file and throws an error on failure."""
-    if not project.tool_settings:
+    if not project.meta:
         raise ProjectError(
             "The pyproject.toml has not been initialized yet. You can do this "
             "by running {}.".format(stream.green("'pdm init'"))
         )
 
 
-def format_toml(pdm_settings):
-    """Ensure the dependencies are inline tables"""
-    for section in pdm_settings:
-        if not section.endswith("dependencies"):
-            continue
-        for name in pdm_settings[section]:
-            if getattr(pdm_settings[section][name], "items", None):
-                table = tomlkit.inline_table()
-                table.update(pdm_settings[section][name])
-                pdm_settings[section][name] = table
-
-
 def find_importable_files(project: Project) -> Iterable[Tuple[str, Path]]:
     """Find all possible files that can be imported"""
     for filename in (
@@ -436,7 +425,7 @@ def set_env_in_reg(env_name: str, value: str) -> None:
                     return
             except FileNotFoundError:
                 old_value, type_ = "", winreg.REG_EXPAND_SZ
-            new_value = ";".join(old_value, value) if old_value else value
+            new_value = os.pathsep.join([old_value, value]) if old_value else value
             try:
                 winreg.SetValueEx(env_key, env_name, 0, type_, new_value)
             except PermissionError:
@@ -453,3 +442,47 @@ def set_env_in_reg(env_name: str, value: str) -> None:
             "please restart the session to take effect."
         )
     )
+
+
+def format_resolution_impossible(err: ResolutionImpossible) -> str:
+    causes: List[RequirementInformation] = err.causes
+    result = []
+    result.append(
+        "Unable to find a resolution that satisfies the following requirements:"
+    )
+
+    for req, parent in causes:
+        result.append(
+            f"  {req.as_line()} (from {repr(parent) if parent else 'project'})"
+        )
+
+    result.append(
+        "Please make sure the package names are correct. If so, you can either "
+        "loosen the version constraints of these dependencies, or "
+        "set a narrower `requires-python` range in the pyproject.toml."
+    )
+    return "\n".join(result)
+
+
+def find_python_in_path(path: os.PathLike) -> str:
+    """Find a python interpreter from the given path, the input argument could be:
+
+    - A valid path to the interpreter
+    - A Python root diretory that contains the interpreter
+    """
+    pathlib_path = Path(path).absolute()
+    if pathlib_path.is_file():
+        return pathlib_path.as_posix()
+
+    if os.name == "nt":
+        for root_dir in (pathlib_path, pathlib_path / "Scripts"):
+            if root_dir.joinpath("python.exe").exists():
+                return root_dir.joinpath("python.exe").as_posix()
+    else:
+        executable_pattern = re.compile(r"python(?:\d(?:\.\d+m?)?)?$")
+
+        for python in pathlib_path.joinpath("bin").glob("python*"):
+            if executable_pattern.match(python.name):
+                return python.as_posix()
+
+    raise NoPythonVersion(f"No Python interpreter is found at {path!r}")
