@@ -8,7 +8,7 @@ import sys
 import sysconfig
 from contextlib import contextmanager
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, List, Optional, Tuple
 
 from distlib.scripts import ScriptMaker
 from pip._internal.req import req_uninstall
@@ -20,16 +20,21 @@ from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
 from pdm.exceptions import NoPythonVersion
 from pdm.iostream import stream
 from pdm.models import pip_shims
+from pdm.models.auth import make_basic_auth
 from pdm.models.builders import EnvBuilder
+from pdm.models.in_process import (
+    get_pep508_environment,
+    get_python_version,
+    get_sys_config_paths,
+)
 from pdm.utils import (
     allow_all_wheels,
     cached_property,
     convert_hashes,
     create_tracked_tempdir,
+    expand_env_vars_in_auth,
     get_finder,
-    get_pep508_environment,
-    get_python_version,
-    get_sys_config_paths,
+    get_python_version_string,
     get_venv_python,
     populate_link,
     temp_environ,
@@ -89,15 +94,16 @@ class Environment:
         self.python_requires = project.python_requires
         self.project = project
         self._essential_installed = False
+        self.auth = make_basic_auth(
+            self.project.sources, stream.verbosity >= stream.DETAIL
+        )
 
     @cached_property
     def python_executable(self) -> str:
         """Get the Python interpreter path."""
         config = self.project.config
-        if config.get("python.path"):
-            return config["python.path"]
-        if PYENV_INSTALLED and config.get("python.use_pyenv", True):
-            return os.path.join(PYENV_ROOT, "shims", "python")
+        if self.project.project_config.get("python.path"):
+            return self.project.project_config["python.path"]
         if "VIRTUAL_ENV" in os.environ:
             stream.echo(
                 "An activated virtualenv is detected, reuse the interpreter now.",
@@ -105,16 +111,18 @@ class Environment:
                 verbosity=stream.DETAIL,
             )
             return get_venv_python(self.project.root)
+        if PYENV_INSTALLED and config.get("python.use_pyenv", True):
+            return os.path.join(PYENV_ROOT, "shims", "python")
 
         # First try what `python` refers to.
         path = shutil.which("python")
         version = None
         if path:
-            version = get_python_version(path, True)
+            version, _ = get_python_version(path, True)
         if not version or not self.python_requires.contains(version):
             finder = Finder()
             for python in finder.find_all_python_versions():
-                version = get_python_version(python.path.as_posix(), True)
+                version, _ = get_python_version(python.path.as_posix(), True)
                 if self.python_requires.contains(version):
                     path = python.path.as_posix()
                     break
@@ -123,6 +131,9 @@ class Environment:
                 if self.python_requires.contains(version):
                     path = sys.executable
         if path:
+            if os.path.normcase(path) == os.path.normcase(sys.executable):
+                # Refer to the base interpreter to allow for venvs
+                path = getattr(sys, "_base_executable", sys.executable)
             stream.echo(
                 "Using Python interpreter: {} ({})".format(stream.green(path), version)
             )
@@ -189,11 +200,18 @@ class Environment:
     @cached_property
     def packages_path(self) -> Path:
         """The local packages path."""
+        version, is_64bit = get_python_version(self.python_executable, True, 2)
         pypackages = (
             self.project.root
             / "__pypackages__"
-            / ".".join(map(str, get_python_version(self.python_executable)[:2]))
+            / get_python_version_string(version, is_64bit)
         )
+        if not pypackages.exists() and not is_64bit:
+            compatible_packages = pypackages.parent / get_python_version_string(
+                version, True
+            )
+            if compatible_packages.exists():
+                pypackages = compatible_packages
         scripts = "Scripts" if os.name == "nt" else "bin"
         for subdir in [scripts, "include", "lib"]:
             pypackages.joinpath(subdir).mkdir(exist_ok=True, parents=True)
@@ -232,7 +250,7 @@ class Environment:
         self,
         sources: Optional[List[Source]] = None,
         ignore_requires_python: bool = False,
-    ) -> pip_shims.PackageFinder:
+    ) -> Generator[pip_shims.PackageFinder, None, None]:
         """Return the package finder of given index sources.
 
         :param sources: a list of sources the finder should search in.
@@ -241,13 +259,15 @@ class Environment:
         if sources is None:
             sources = self.project.sources
         sources = sources or []
-        python_version = get_python_version(self.python_executable)[:2]
+        python_version, _ = get_python_version(self.python_executable, digits=2)
         finder = get_finder(
             sources,
             self.project.cache_dir.as_posix(),
             python_version,
             ignore_requires_python,
         )
+        # Reuse the auth across sessions to avoid prompting repeatly.
+        finder.session.auth = self.auth
         yield finder
         finder.session.close()
 
@@ -277,7 +297,10 @@ class Environment:
                         ireq.req.project_name,
                         pip_shims.get_supported(
                             version="".join(
-                                map(str, get_python_version(self.python_executable)[:2])
+                                map(
+                                    str,
+                                    get_python_version(self.python_executable)[0][:2],
+                                )
                             )
                         ),
                     )
@@ -300,6 +323,8 @@ class Environment:
                 ireq.hash_options = convert_hashes(hashes)
             if not (ireq.editable and ireq.req.is_local_dir):
                 downloader = pip_shims.Downloader(finder.session, "off")
+                if ireq.link.is_vcs:
+                    ireq.link = pip_shims.Link(expand_env_vars_in_auth(ireq.link.url))
                 downloaded = pip_shims.unpack_url(
                     ireq.link,
                     ireq.source_dir,
@@ -313,10 +338,26 @@ class Environment:
                         shutil.copy(downloaded.path, download_dir)
                     except shutil.SameFileError:
                         pass
-            # Now all source is prepared, build it.
-            if ireq.link.is_wheel:
-                return (self.project.cache("wheels") / ireq.link.filename).as_posix()
 
+            if ireq.link.is_wheel:
+                # If the file is a wheel, should be already present under download dir.
+                return (self.project.cache("wheels") / ireq.link.filename).as_posix()
+            else:
+                # Check the built wheel cache again after hashes are resolved.
+                cache_entry = wheel_cache.get_cache_entry(
+                    ireq.link,
+                    ireq.req.project_name,
+                    pip_shims.get_supported(
+                        version="".join(
+                            map(str, get_python_version(self.python_executable)[0][:2])
+                        )
+                    ),
+                )
+                if cache_entry is not None:
+                    stream.logger.debug("Using cached wheel link: %s", cache_entry.link)
+                    return cache_entry.link.file_path
+
+            # Otherwise, now all source is prepared, build it.
             with EnvBuilder(ireq.unpacked_source_directory, self) as builder:
                 if ireq.editable:
                     ret = builder.build_egg_info(kwargs["build_dir"])
@@ -349,7 +390,7 @@ class Environment:
         """Get the working set based on local packages directory."""
         paths = self.get_paths()
         return WorkingSet(
-            [paths["platlib"]], python=get_python_version(self.python_executable)
+            [paths["platlib"]], python=get_python_version(self.python_executable)[0]
         )
 
     @cached_property
@@ -362,7 +403,7 @@ class Environment:
         if not os.path.isabs(command) and command.startswith("python"):
             python = os.path.splitext(command)[0]
             version = python[6:]
-            this_version = get_python_version(self.python_executable, True)
+            this_version, _ = get_python_version(self.python_executable, True)
             if not version or this_version.startswith(version):
                 return self.python_executable
         # Fallback to use shutil.which to find the executable
@@ -376,7 +417,7 @@ class Environment:
         scripts = self.get_paths()["scripts"]
         maker = ScriptMaker(None, None)
         maker.executable = new_path
-        shebang = maker._get_shebang("utf-8").rstrip()
+        shebang = maker._get_shebang("utf-8").rstrip().replace(b"\\", b"\\\\")
         for child in Path(scripts).iterdir():
             if not child.is_file() or child.suffix not in (".exe", ".py", ""):
                 continue
