@@ -19,7 +19,7 @@ def get_specifier(version_str: Union[SpecifierSet, str]) -> SpecifierSet:
 def parse_version_tuple(version: str) -> Tuple[Union[int, str], ...]:
     version = re.sub(r"(?<!\.)\*", ".*", version)
     try:
-        return tuple(int(v) if v != "*" else v for v in version.split("."))
+        return tuple(int(v) if v != "*" else v for v in version.split("."))[:3]
     except ValueError:
         raise InvalidPyVersion(
             f"{version}: Prereleases or postreleases are not supported "
@@ -79,6 +79,8 @@ class PySpecSet(SpecifierSet):
     MAX_VERSION = (99, 99, 99)
 
     def __init__(self, version_str: str = "", analyze: bool = True) -> None:
+        if version_str == "*":
+            version_str = ""
         super().__init__(version_str)
         self._lower_bound = self.MIN_VERSION  # type: Tuple[int, int, int]
         self._upper_bound = self.MAX_VERSION  # type: Tuple[int, int, int]
@@ -414,7 +416,7 @@ class PySpecSet(SpecifierSet):
         return (
             lower <= self._lower_bound
             and upper >= self._upper_bound
-            and set(excludes) >= set(self._excludes)
+            and set(self._excludes) >= set(excludes)
         )
 
     def as_marker_string(self) -> str:
