@@ -79,6 +79,12 @@ class Config(MutableMapping):
             True,
             "PDM_AUTO_GLOBAL",
         ),
+        "strategy.update": ConfigItem(
+            "The default strategy for updating packages", "reuse", False
+        ),
+        "strategy.save": ConfigItem(
+            "Specify how to save versions when a package is added", "compatible", False
+        ),
         "parallel_install": ConfigItem(
             "Whether to perform installation and uninstallation in parallel",
             True,
@@ -134,22 +140,25 @@ class Config(MutableMapping):
             *parts, last = key.split(".")
             temp = toml_data
             for part in parts:
-                temp = temp.setdefault(part, {})
+                if part not in temp:
+                    temp[part] = {}
+                temp = temp[part]
             temp[last] = value
 
         with self._config_file.open("w", encoding="utf-8") as fp:
             fp.write(tomlkit.dumps(toml_data))
 
     def __getitem__(self, key: str) -> Any:
-        if key not in self._data:
-            raise NoConfigError(key)
         env_var = self._config_map[key].env_var
         if env_var is not None and env_var in os.environ:
             env_value = os.environ[env_var]
             if isinstance(self._config_map[key].default, bool):
                 env_value = ensure_boolean(env_value)
             return env_value
-        return self._data[key]
+        try:
+            return self._data[key]
+        except KeyError:
+            raise NoConfigError(key) from None
 
     def __setitem__(self, key: str, value: Any) -> None:
         if key not in self._config_map:
@@ -168,7 +177,7 @@ class Config(MutableMapping):
             stream.echo(
                 stream.yellow(
                     "WARNING: the config is shadowed by env var '{}', "
-                    "set value won't take effect.".format(env_var)
+                    "the value set won't take effect.".format(env_var)
                 )
             )
         self._data[key] = value
