@@ -2,20 +2,19 @@
 Utility functions
 """
 import atexit
-import functools
-import importlib
-import json
 import os
+import re
 import shutil
 import subprocess
 import tempfile
 import urllib.parse as parse
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Generic, List, Optional, Tuple, TypeVar, Union
 
 from distlib.wheel import Wheel
-from packaging.version import parse as parse_version
+from tomlkit.container import Container
+from tomlkit.items import Item
 
 from pdm._types import Source
 from pdm.models.pip_shims import (
@@ -30,13 +29,15 @@ try:
     from functools import cached_property
 except ImportError:
 
-    class cached_property:
-        def __init__(self, func):
+    _T = TypeVar("_T")
+
+    class cached_property(Generic[_T]):
+        def __init__(self, func: Callable[[Any], _T]):
             self.func = func
             self.attr_name = func.__name__
             self.__doc__ = func.__doc__
 
-        def __get__(self, inst, cls=None):
+        def __get__(self, inst: Any, cls=None) -> _T:
             if inst is None:
                 return self
             if self.attr_name not in inst.__dict__:
@@ -190,47 +191,6 @@ def find_project_root(cwd: str = ".", max_depth: int = 5) -> Optional[str]:
     return None
 
 
-@functools.lru_cache()
-def get_python_version(executable, as_string=False):
-    """Get the version of the Python interperter."""
-    args = [
-        executable,
-        "-c",
-        "import sys,json;print(json.dumps(tuple(sys.version_info[:3])))",
-    ]
-    result = tuple(json.loads(subprocess.check_output(args)))
-    if not as_string:
-        return result
-    return ".".join(map(str, result))
-
-
-def get_sys_config_paths(executable: str, vars=None) -> Dict[str, str]:
-    """Return the sys_config.get_paths() result for the python interpreter"""
-    if not vars:
-        args = [
-            executable,
-            "-c",
-            "import sysconfig,json;print(json.dumps(sysconfig.get_paths()))",
-        ]
-        return json.loads(subprocess.check_output(args))
-    else:
-        env = os.environ.copy()
-        env.update(SYSCONFIG_VARS=json.dumps(vars))
-        args = [
-            executable,
-            "-c",
-            "import os,sysconfig,json;print(json.dumps(sysconfig."
-            "get_paths(vars=json.loads(os.getenv('SYSCONFIG_VARS')))))",
-        ]
-        return json.loads(subprocess.check_output(args, env=env))
-
-
-def get_pep508_environment(executable: str) -> Dict[str, Any]:
-    script = importlib.import_module("pdm.pep508").__file__.rstrip("co")
-    args = [executable, script]
-    return json.loads(subprocess.check_output(args))
-
-
 def convert_hashes(hashes: Dict[str, str]) -> Dict[str, List[str]]:
     """Convert Pipfile.lock hash lines into InstallRequirement option format.
 
@@ -243,7 +203,7 @@ def convert_hashes(hashes: Dict[str, str]) -> Dict[str, List[str]]:
             name, hash_value = hash_value.split(":")
         except ValueError:
             name = "sha256"
-        result.setdefault(name, []).append(hash_value)
+        setdefault(result, name, []).append(hash_value)
     return result
 
 
@@ -367,11 +327,6 @@ def open_file(url, session=None):
                 result.close()
 
 
-def highest_version(versions: List[str]) -> str:
-    """Return the highest version of a given list."""
-    return max(versions, key=parse_version)
-
-
 def populate_link(
     finder: PackageFinder,
     ireq: InstallRequirement,
@@ -384,3 +339,42 @@ def populate_link(
             return
         link = getattr(link, "link", link)
         ireq.link = link
+
+
+def setdefault(document: Container, key: str, value: Any) -> Item:
+    """A compatiable dict.setdefault() for tomlkit data structures."""
+    if key not in document:
+        document[key] = value
+    return document[key]
+
+
+def get_python_version_string(version: str, is_64bit: bool) -> str:
+    if os.name == "nt" and not is_64bit:
+        return f"{version}-32"
+    return version
+
+
+def expand_env_vars(credential: str, quote: bool = False) -> str:
+    """A safe implementation of env var substitution.
+    It only supports the following forms:
+
+        ${ENV_VAR}
+
+    Neither $ENV_VAR and %ENV_VAR is not supported.
+    """
+
+    def replace_func(match):
+        rv = os.getenv(match.group(1), match.group(0))
+        return parse.quote(rv) if quote else rv
+
+    return re.sub(r"\$\{(.+?)\}", replace_func, credential)
+
+
+def expand_env_vars_in_auth(url: str) -> str:
+    """In-place expand the auth in url"""
+    scheme, netloc, path, params, query, fragment = parse.urlparse(url)
+    if "@" in netloc:
+        auth, rest = netloc.split("@", 1)
+        auth = expand_env_vars(auth, True)
+        netloc = "@".join([auth, rest])
+    return parse.urlunparse((scheme, netloc, path, params, query, fragment))
