@@ -6,8 +6,9 @@ from pathlib import Path
 import pytest
 
 from pdm.cli import actions
+from pdm.models.in_process import get_python_version
 from pdm.models.requirements import parse_requirement
-from pdm.utils import get_python_version, temp_environ
+from pdm.utils import temp_environ
 from tests import FIXTURES
 
 
@@ -110,7 +111,7 @@ def test_use_command(project, invoke):
     result = invoke(["use", "-f", python_path], obj=project)
     assert result.exit_code == 0
 
-    project.tool_settings["python_requires"] = ">=3.6"
+    project.meta["requires-python"] = ">=3.6"
     project.write_pyproject()
     result = invoke(["use", "2.7"], obj=project)
     assert result.exit_code == 1
@@ -145,8 +146,8 @@ def test_init_command(project_no_init, invoke, mocker):
         ["init"], input="python\ntest-project\n\n\n\n\n\n", obj=project_no_init
     )
     assert result.exit_code == 0
-    python_version = ".".join(
-        map(str, get_python_version(project_no_init.environment.python_executable)[:2])
+    python_version, _ = get_python_version(
+        project_no_init.environment.python_executable, True, 2
     )
     do_init.assert_called_with(
         project_no_init,
@@ -236,9 +237,17 @@ def test_cache_clear_command(project, invoke, mocker):
 def test_import_other_format_file(project, invoke, filename):
     requirements_file = FIXTURES / filename
     result = invoke(["import", str(requirements_file)], obj=project)
+    print(result.stderr)
     assert result.exit_code == 0
 
 
+@pytest.mark.pypi
+def test_search_package(project, invoke):
+    result = invoke(["search", "requests"], obj=project)
+    assert result.exit_code == 0
+    assert len(result.output.splitlines()) > 0
+
+
 @pytest.mark.pypi
 def test_show_package_on_pypi(invoke):
     result = invoke(["show", "ipython"])
@@ -281,3 +290,10 @@ def test_completion_command(invoke):
     result = invoke(["completion", "bash"])
     assert result.exit_code == 0
     assert "(completion)" in result.output
+
+
+def test_lock_legacy_project(invoke, fixture_project, repository):
+    project = fixture_project("demo-legacy")
+    result = invoke(["lock"], obj=project)
+    assert result.exit_code == 0
+    assert "urllib3" in project.get_locked_candidates()
