@@ -229,7 +229,7 @@ def project_no_init(tmp_path, mocker):
     mocker.patch("pdm.project.core.Config.HOME_CONFIG", tmp_path)
     old_config_map = Config._config_map.copy()
     p.global_config["cache_dir"] = tmp_path.joinpath("caches").as_posix()
-    do_use(p, sys.executable)
+    do_use(p, getattr(sys, "_base_executable", sys.executable))
     with temp_environ():
         os.environ.pop("VIRTUAL_ENV", None)
         os.environ.pop("PYTHONPATH", None)
@@ -253,6 +253,7 @@ def fixture_project(project_no_init):
     def func(project_name):
         source = FIXTURES / "projects" / project_name
         copy_tree(source.as_posix(), project_no_init.root.as_posix())
+        project_no_init._pyproject = None
         return project_no_init
 
     return func
